@@ -384,9 +384,9 @@ partition_setup(struct disk *dp)
   hd_name[8] = '\0';			/* i.e.  /dev/hda6 -> /dev/hda */
 #endif
 
-  priv_on();
+  enter_priv_on();
   part_fd = DOS_SYSCALL(open(hd_name, O_RDONLY));
-  priv_default();
+  leave_priv_setting();
   if (part_fd == -1) {
     error("ERROR: opening device %s to read MBR for PARTITION %s\n",
 	  hd_name, dp->dev_name);
@@ -547,14 +547,14 @@ disk_open(struct disk *dp)
   if (dp == NULL || dp->fdesc >= 0)
     return;
     
-  priv_on();
+  enter_priv_on();
   dp->fdesc = DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
-  priv_default();
+  leave_priv_setting();
   if (dp->fdesc < 0) 
     if (errno == EROFS || errno == EACCES) {
-      priv_on();
+      enter_priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
-      priv_default();
+      leave_priv_setting();
       if (dp->fdesc < 0) {
         d_printf("ERROR: (disk) can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
         /* In case we DO get more clever, we want to share that code */
@@ -618,15 +618,15 @@ disk_open(struct disk *dp)
   if (dp == NULL || dp->fdesc >= 0)
     return;
     
-  priv_on();
-  dp->fdesc = DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
-  priv_default();
+  enter_priv_on();
+  dp->fdesc = SILENT_DOS_SYSCALL(open(dp->dev_name, dp->wantrdonly ? O_RDONLY : O_RDWR, 0));
+  leave_priv_setting();
 
   if (dp->fdesc < 0) 
-    if (errno == EROFS) {
-      priv_on();
+    if (errno == EROFS || errno == ENODEV) {
+      enter_priv_on();
       dp->fdesc = DOS_SYSCALL(open(dp->dev_name, O_RDONLY, 0));
-      priv_default();
+      leave_priv_setting();
       if (dp->fdesc < 0) {
         d_printf("ERROR: (disk) can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
         /* In case we DO get more clever, we want to share that code */
@@ -729,15 +729,15 @@ disk_init(void)
 #endif
 
   if (config.bootdisk) {
-    priv_on();
+    enter_priv_on();
     bootdisk.fdesc = open(bootdisk.dev_name,
 			  bootdisk.rdonly ? O_RDONLY : O_RDWR, 0);
-    priv_default();
+    leave_priv_setting();
     if (bootdisk.fdesc < 0) 
       if (errno == EROFS) {
-        priv_on();
+        enter_priv_on();
         bootdisk.fdesc = open(bootdisk.dev_name, O_RDONLY, 0);
-        priv_default();
+        leave_priv_setting();
         if (bootdisk.fdesc < 0) {
           error("ERROR: can't open bootdisk %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(23);
@@ -779,14 +779,14 @@ disk_init(void)
       continue;
     }
 #endif
-    priv_on();
+    enter_priv_on();
     dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
-    priv_default();
+    leave_priv_setting();
     if (dp->fdesc < 0) 
       if (errno == EROFS || errno == EACCES) {
-        priv_on();
+        enter_priv_on();
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
-        priv_default();
+        leave_priv_setting();
         if (dp->fdesc < 0) {
           error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(25);
@@ -803,16 +803,16 @@ disk_init(void)
   for (dp = hdisktab; dp < &hdisktab[HDISKS]; dp++) {
     if(dp->type == IMAGE)  {
 	d_printf("IMAGE: Using user permissions\n");
-	priv_off();
+	enter_priv_off();
     }
-    else priv_on();
+    else enter_priv_on();
     dp->fdesc = open(dp->dev_name, dp->rdonly ? O_RDONLY : O_RDWR, 0);
-    priv_default();
+    leave_priv_setting();
     if (dp->fdesc < 0) {
       if (errno == EROFS || errno == EACCES) {
-        if (dp->type == IMAGE) priv_off(); else priv_on();
+        if (dp->type == IMAGE) enter_priv_off(); else enter_priv_on();
         dp->fdesc = open(dp->dev_name, O_RDONLY, 0);
-        priv_default();
+        leave_priv_setting();
         if (dp->fdesc < 0) {
           error("ERROR: can't open %s for read nor write: %s (you should never see this message)\n", dp->dev_name, strerror(errno));
           leavedos(26);
